﻿using Battleship.Drawings;
using Battleship.MovementStrategy;

namespace Battleship;
/// <summary>
/// Форма работы с объектом Линкор
/// </summary>
public partial class FormBattleship : Form
{
    /// <summary>
    /// Поле/объект для прорисовки объекта
    /// </summary>
    private DrawingWarship? _drawingWarship; //поля с нижнего подчеркивания

    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;

    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormBattleship()
    {
        InitializeComponent();
        _strategy = null;
    }
    /// <summary>
    /// Метод прорисовки машины
    /// </summary>
    //создаем отдельный метод
    private void Draw()
    {
        if (_drawingWarship == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxBattleship.Width, pictureBoxBattleship.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawingWarship.DrawTransport(gr);
        pictureBoxBattleship.Image = bmp;

    }

    private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawingWarship):
                _drawingWarship = new DrawingWarship(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawingBattleship):
                _drawingWarship = new DrawingBattleship(random.Next(100, 300), random.Next(1000, 3000),
                Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
                Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }

        _drawingWarship.SetPictureSize(pictureBoxBattleship.Width, pictureBoxBattleship.Height);
        _drawingWarship.SetPosition(random.Next(10, 100), random.Next(10, 100), pictureBoxBattleship.Width, pictureBoxBattleship.Height);
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }

    /// <summary>
    /// обработка нажатия кнопки "создать линкор"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonCreate_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawingBattleship));

    }

    /// <summary>
    /// обработка нажатия кнопки "создать военный корабль"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonCreateWarshit_Click(object sender, EventArgs e)
    {
        CreateObject(nameof(DrawingWarship));

    }

    /// <summary>
    /// кнопки вверх/вниз/влево/вправо 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void buttonMove_Click(object sender, EventArgs e)
    {
        if (_drawingWarship == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawingWarship.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawingWarship.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawingWarship.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawingWarship.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    private void buttonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawingWarship == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex
            switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };

            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableWarship(_drawingWarship),
            pictureBoxBattleship.Width, pictureBoxBattleship.Height);
        }

        if (_strategy == null)
        {
            return;
        }
        comboBoxStrategy.Enabled = false;
         _strategy.MakeStep();
         Draw();

         if (_strategy.GetStatus() == StrategyStatus.Finish)
         {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
         }
    }
}